import os
import glob

import unsonic
from . import Command, register


@register
class Man(Command):
    NAME = "man"
    HELP = "Show the manpages for Unsonic."
    DESC = HELP
    INSTALL = os.path.join(unsonic.INSTALL, "docs/man")
    CFG_NEEDED = False


    def _initArgParser(self, parser):
        parser.add_argument("-l", "--list", action="store_true",
                            help="List the man pages")
        parser.add_argument("page", nargs='?',
                            help="The man page to view, defaults to 'unsonic'")


    def run(self, args, config):
        super()._run()
        if args.list:
            files = glob.glob(os.path.join(self.INSTALL, "*.1"))
            files = [".".join(f.split(".")[:-1]) for f in files]
            files = "  ".join([os.path.basename(f) for f in files])
            print(files)
            return

        page = args.page if args.page else "unsonic"
        f = os.path.join(self.INSTALL, page) + ".1"
        if os.path.exists(f):
            os.system("man " + f)
        else:
            print("Invalid man page: " + page)



# flake8: noqa: F401

# Location information
import os
import sys
HERE = os.getcwd()
INSTALL = "/".join(os.path.dirname(__file__).split("/"))
CMD = os.path.abspath(sys.argv[0])
del os
del sys


# This needs to be very early in the import sequence so the monkeypatch
# works right.
from . import config

from . import models, log, auth


# Used by pkg_resources and Pyramid to load the webapp
from .web import main, webServe



# flake8: noqa: F401

import sys

from nicfit.command import register
from mishmash import core

from unsonic import __main__


class Command(core.Command):
    CFG_NEEDED = True
    
    def _run(self, args=None):
        if self.CFG_NEEDED:
            if not __main__.APP.cfg_found:
                print("Could not find a standardly located config. "
                      "You must specify the config file with -c argument, "
                      "example: unsonic -c /etc/unsonic.ini ...")
                sys.exit(-1)
            initAlembic(self.config.get("mishmash", "sqlalchemy.url"))


from ..models import initAlembic

from .adduser import AddUser
from .config import Config
from .deluser import DelUser
from .listusers import ListUsers
from .password import Password
from .serve import Serve
from .man import Man
from .install import Install



import sys
import argparse
import unsonic
import unsonic.commands         # noqa: F401
from unsonic import version, config
from nicfit import Command

import mishmash
from mishmash.__main__ import MishMash
from mishmash import __about__


VERSION = "unsonic (%s, protocol: %s, subsonic protocol: %s) mishmash (%s)" % \
          (version.VERSION, version.UNSONIC_PROTOCOL_VERSION,
           version.PROTOCOL_VERSION, __about__.__version__)
APP = None


class Unsonic(MishMash):

    def __init__(self):
        super().__init__(progname="unsonic")
        self.cfg_found = False
        self._orig_main_func = self._main_func
        self._main_func = self.mainWrapper


    def mainWrapper(self, args):
        # Handle mishmash commands which assume a default config
        # Unsonic commands are handled in the unsonic base command class
        need_cfg = False
        if (hasattr(self.args, "command_func") and
            hasattr(self.args.command_func, "__self__")):
            need_cfg = ("mishmash.commands" in
                        self.args.command_func.__self__.__module__)
        if not self.cfg_found and need_cfg:
            print("Could not find a standardly located config. "
                  "You must specify the config file with -c argument, "
                  "example: unsonic -c /etc/unsonic.ini ...")
            sys.exit(-1)

        return self._orig_main_func(args)


    # hack in the unsonic version
    def _addArguments(self, parser):
        super()._addArguments(parser)
        for index in range(len(parser._actions)):
            if isinstance(parser._actions[index], argparse._VersionAction):
                parser._actions[index].version = VERSION
                break


def buildApp():
    global APP
    if "web" in Command._all_commands:
        del Command._all_commands["web"]
    if mishmash.commands.web.Web.name in Command._all_commands:
        del Command._all_commands[mishmash.commands.web.Web.name]
    APP = Unsonic()
    return APP


def adjustCmdline(parser):
    path = config.findConfig(parser)
    if path is False:
        APP.cfg_found = False
    elif path is not True:
        # Append the found config file
        sys.argv = sys.argv[:1] + ["-c", path] + sys.argv[1:]
        APP.cfg_found = True
    else:
        # its already supplied, just carry on
        APP.cfg_found = True


def run(args=None):
    app = buildApp()
    adjustCmdline(app.arg_parser)
    return app.run(args)


def main(args=None):
    app = buildApp()
    adjustCmdline(app.arg_parser)
    return app.main(args)


if __name__ == "__main__":
    run()



import os

import unsonic
from . import Command, register


###
### To system packagers (if there are any..)
###
### Feel free to disable this command as this stuff should be handled in the
### package. Just remove the import of install in this directories __init__.py.
###


@register
class Install(Command):
    NAME = "install"
    HELP = "Install the Unsonic service."
    DESC = "Install the Unsonic service. Requires root permissions."
    CFG_NEEDED = False

    def _initArgParser(self, parser):
        parser.add_argument("-u", "--user", default="unsonic",
                            help="User to run unsonic as. default: unsonic")
        parser.add_argument("-r", "--rundir", default="/var/lib/unsonic",
                            help=("Where unsonic's files are located. "
                                  "default: /var/lib/unsonic"))


    def run(self, args, config):
        super()._run()
        cmd = "/bin/bash %s %s %s %s %s" % (
            os.path.join(unsonic.INSTALL, "etc/install.sh"),
            unsonic.CMD, unsonic.INSTALL, args.user, args.rundir)

        os.system(cmd)



import os
import configparser
import logging

# This needs to be very early in the import sequence so the monkeypatch
# works right.
import mishmash
from mishmash.config import Config as MishConfig

from unsonic import HERE, INSTALL


import nicfit
log = nicfit.getLogger(__name__)


CONFIG = None
DEF_LOCS = [
    # Production paths
    "/etc/unsonic.ini",
    os.path.join(INSTALL, "etc/unsonic.ini"),
    os.path.join(HERE, "unsonic.ini"),
    os.path.join(HERE, "unsonic.ini"),
    # Dev paths
    os.path.join(HERE, "development.ini"),
    os.path.join(HERE, "unsonic/etc/development.ini")
    ]


class HereConfig(MishConfig):
    def __init__(self, filename, **kwargs):
        super().__init__(filename, **kwargs)
        global CONFIG
        CONFIG = self
        # %(install)s
        self.set(configparser.DEFAULTSECT, "install", INSTALL)
        # %(here)s
        if filename:
            here = os.path.dirname(os.path.abspath(filename))
            self.set(configparser.DEFAULTSECT, "here", here)
        else:
            self.set(configparser.DEFAULTSECT, "here", INSTALL)
        # %(venv)s
        if "VIRTUAL_ENV" in os.environ:
            self.set(configparser.DEFAULTSECT, "venv",
                     os.path.abspath(os.environ["VIRTUAL_ENV"]))
        else:
            self.set(configparser.DEFAULTSECT, "venv",
                     os.path.abspath("venv"))


    def get(self, section, key, **kwargs):
        val = super().get(section, key, **kwargs)
        if val and "%(here)s" in val:
            return collapseRelativePaths(
                val.replace("%(here)s", super().get(configparser.DEFAULTSECT,
                                                    "here")))
        elif val and "%(install)s" in val:
            return collapseRelativePaths(
                val.replace("%(install)s", super().get(configparser.DEFAULTSECT,
                                                       "install")))
        elif val and "%(venv)s" in val:
            return collapseRelativePaths(
                val.replace("%(venv)s", super().get(configparser.DEFAULTSECT,
                                                       "venv")))
        else:
            return val

    def here(self):
        return self.get(configparser.DEFAULTSECT, "here")

    def install(self):
        return self.get(configparser.DEFAULTSECT, "install")

    def venv(self):
        return self.get(configparser.DEFAULTSECT, "venv")


mishmash.config.Config = HereConfig


def collapseRelativePaths(path):
    output = []
    skip = 0
    bits = path.split("/")
    bits.reverse()
    for bit in bits:
        if bit == "..":
            skip += 1
        else:
            if skip > 0:
                skip -= 1
                continue
            else:
                output.append(bit)
    output.reverse()
    return "/".join(output)


# Try to find a standard config file if not specified on the cmdline
def findConfig(parser):
    args, unknown = parser.parse_known_args()
    if args.config is not parser.get_default("config"):
        log.debug("Using user specified config file")
        return True

    for path in DEF_LOCS:
        if os.path.isfile(path):
            log.debug("No config specified, found config file: " +
                      os.path.abspath(path))
            return path
    else:
        return False
